@@ -32,7 +32,7 @@
 /// \endcond
 
 // Additional local function prototypes
-int sun_eph(double jd, double *ra, double *dec, double *dis);
+int sun_eph(double jd, double *restrict ra, double *restrict dec, double *restrict dis);
 
 /**
  * Whether the high-precision call is allowed to return a low-precision result. If set to 0
@@ -99,8 +99,8 @@ void enable_earth_sun_hp(int value) {
  * @sa novas_planet_provider
  *
  */
-short earth_sun_calc(double jd_tdb, enum novas_planet body, enum novas_origin origin, double *position,
-        double *velocity) {
+short earth_sun_calc(double jd_tdb, enum novas_planet body, enum novas_origin origin,
+        double *restrict position, double *restrict velocity) {
   static const char *fn = "earth_sun_calc";
 
   // The arrays below contain masses and orbital elements for the four
@@ -148,12 +148,17 @@ short earth_sun_calc(double jd_tdb, enum novas_planet body, enum novas_origin or
 
       // Compute p and q vectors (see Brouwer & Clemence (1961), Methods of
       // Celestial Mechanics, pp. 35-36.)
+      const double t1 = cw * sn + sw * cn * ci;
+      const double t2 = sw * si;
       const double p1 = cw * cn - sw * sn * ci;
-      const double p2 = (cw * sn + sw * cn * ci) * ce - sw * si * se;
-      const double p3 = (cw * sn + sw * cn * ci) * se + sw * si * ce;
+      const double p2 = t1 * ce - t2 * se;
+      const double p3 = t1 * se + t2 * ce;
+
+      const double ta = cw * cn * ci - sw * sn;
+      const double tb = cw * si;
       const double q1 = -sw * cn - cw * sn * ci;
-      const double q2 = (-sw * sn + cw * cn * ci) * ce - cw * si * se;
-      const double q3 = (-sw * sn + cw * cn * ci) * se + cw * si * ce;
+      const double q2 = ta * ce - tb * se;
+      const double q3 = ta * se + tb * ce;
       const double roote = sqrt(1.0 - pe[i] * pe[i]);
 
       a[0][i] = pa[i] * p1;
@@ -293,18 +298,18 @@ short earth_sun_calc(double jd_tdb, enum novas_planet body, enum novas_origin or
  * @sa novas_planet_provider_hp
  *
  */
-short earth_sun_calc_hp(const double jd_tdb[2], enum novas_planet body, enum novas_origin origin, double *position,
-        double *velocity) {
+short earth_sun_calc_hp(const double jd_tdb[restrict 2], enum novas_planet body, enum novas_origin origin,
+        double *restrict position, double *restrict velocity) {
   static const char *fn = "earth_sun_calc_hp";
 
   if(!jd_tdb)
     return novas_error(-1, EINVAL, fn, "NULL jd_tdb parameter");
 
-  prop_error(fn, earth_sun_calc(jd_tdb[0] + jd_tdb[1], body, origin, position, velocity), 0);
-
   if(!allow_lp_for_hp)
     return novas_error(3, EAGAIN, fn, "low-precision calculation is not currently allowed as a substitute");
 
+  prop_error(fn, earth_sun_calc(jd_tdb[0] + jd_tdb[1], body, origin, position, velocity), 0);
+
   return 0;
 }
 
@@ -335,7 +340,7 @@ short earth_sun_calc_hp(const double jd_tdb[2], enum novas_planet body, enum nov
  *
  * @sa earth_sun_calc()
  */
-int sun_eph(double jd, double *ra, double *dec, double *dis) {
+int sun_eph(double jd, double *restrict ra, double *restrict dec, double *restrict dis) {
   struct sun_con {
     int l;
     int r;
@@ -434,7 +439,7 @@ int sun_eph(double jd, double *ra, double *dec, double *dis) {
   sin_lon = sin(lon);
   *ra = atan2((cos(emean) * sin_lon), cos(lon)) * RAD2DEG;
   if(*ra < 0.0)
-    *ra += 360.0;
+    *ra += DEG360;
   *ra = *ra / 15.0;
 
   *dec = asin(sin(emean) * sin_lon) * RAD2DEG;
@@ -448,12 +453,12 @@ novas_planet_provider planet_call = earth_sun_calc;
 novas_planet_provider_hp planet_call_hp = earth_sun_calc_hp;
 /// \endcond
 #elif !BUILTIN_SOLSYS3
-short solarsystem(double jd_tdb, short body, short origin, double *position, double *velocity) {
+short solarsystem(double jd_tdb, short body, short origin, double *restrict position, double *restrict velocity) {
   prop_error("solarsystem", earth_sun_calc(jd_tdb, body, origin, position, velocity), 0);
   return 0;
 }
 
-short solarsystem_hp(const double jd_tdb[2], short body, short origin, double *position, double *velocity) {
+short solarsystem_hp(const double jd_tdb[restrict 2], short body, short origin, double *restrict position, double *restrict velocity) {
   prop_error("solarsystem_hp", earth_sun_calc_hp(jd_tdb, body, origin, position, velocity), 0);
   return 0;
 }
