@@ -13,6 +13,23 @@
 #define __NOVAS_INTERNAL_API__      ///< Use definitions meant for internal use by SuperNOVAS only
 #include "novas.h"
 
+static int dummy_ephem(const char *name, long id, double jd_tdb_high, double jd_tdb_low, enum novas_origin *origin, double *pos, double *vel) {
+  (void) name;
+  (void) jd_tdb_high;
+  (void) jd_tdb_low;
+  *origin = NOVAS_BARYCENTER;
+
+  if(pos) {
+    memset(pos, 0, 3 * sizeof(double));
+    pos[0] = id % 100;
+  }
+  if(vel) {
+    memset(vel, 0, 3 * sizeof(double));
+    vel[1] = 0.01 * (id % 100);
+  }
+  return 0;
+}
+
 static int check(const char *func, int exp, int error) {
   if(error != exp) {
     fprintf(stderr, "ERROR! %s: expected %d, got %d\n", func, exp, error);
@@ -35,7 +52,7 @@ static int test_make_on_surface() {
 }
 
 static int test_make_in_space() {
-  double p[3] = {1.0}, v[3] = {};
+  double p[3] = {1.0}, v[3] = {0.0};
   in_space sp;
   int n = 0;
 
@@ -47,9 +64,9 @@ static int test_make_in_space() {
 }
 
 static int test_make_observer() {
-  in_space sp = {};
-  on_surface on = {};
-  observer obs = {};
+  in_space sp = IN_SPACE_INIT;
+  on_surface on = ON_SURFACE_INIT;
+  observer obs = OBSERVER_INIT;
   int n = 0;
 
   if(check("make_observer:where", 1, make_observer(-1, &on, &sp, &obs))) n++;
@@ -63,8 +80,8 @@ static int test_make_observer() {
 }
 
 static int test_make_airborne_observer() {
-  on_surface on = {};
-  observer obs = {};
+  on_surface on = ON_SURFACE_INIT;
+  observer obs = OBSERVER_INIT;
   int n = 0;
 
   if(check("make_airborne_observer:vel", -1, make_airborne_observer(&on, NULL, &obs))) n++;
@@ -113,8 +130,8 @@ static int test_make_cat_entry() {
 }
 
 static int test_make_object() {
-  cat_entry s = {};
-  object o = {};
+  cat_entry s = CAT_ENTRY_INIT;
+  object o = NOVAS_OBJECT_INIT;
   int n = 0;
 
   char longname[SIZE_OF_OBJ_NAME + 1];
@@ -168,7 +185,7 @@ static int test_z2v() {
 }
 
 static int test_refract() {
-  on_surface o = {};
+  on_surface o = ON_SURFACE_INIT;
   int n = 0;
 
   novas_debug(NOVAS_DEBUG_ON);
@@ -191,7 +208,7 @@ static int test_refract() {
 
 static int test_refract_astro() {
   extern int novas_inv_max_iter;
-  on_surface surf = {};
+  on_surface surf = ON_SURFACE_INIT;
   int n = 0;
 
   novas_inv_max_iter = 0;
@@ -205,7 +222,7 @@ static int test_refract_astro() {
 
 static int test_inv_refract() {
   extern int novas_inv_max_iter;
-  on_surface surf = {};
+  on_surface surf = ON_SURFACE_INIT;
   int n = 0;
 
   novas_inv_max_iter = 0;
@@ -217,7 +234,7 @@ static int test_inv_refract() {
 }
 
 static int test_limb_angle() {
-  double pos[3] = { 0.01 }, pn[3] = { -0.01 }, pz[3] = {}, a, b;
+  double pos[3] = { 0.01 }, pn[3] = { -0.01 }, pz[3] = {0.0}, a, b;
   int n = 0;
 
   if(check("limb_angle:pos_obj", -1, limb_angle(NULL, pos, &a, &b))) n++;
@@ -237,11 +254,11 @@ static int test_limb_angle() {
 
 
 static int test_transform_cat() {
-  cat_entry c = {}, c1;
+  cat_entry c = CAT_ENTRY_INIT, c1;
   int n = 0;
 
-  char longname[SIZE_OF_OBJ_NAME + 1];
-  memset(longname, 'A', SIZE_OF_OBJ_NAME);
+  char longname[SIZE_OF_CAT_NAME + 1] = {'\0'};
+  memset(longname, 'A', SIZE_OF_CAT_NAME);
 
   if(check("transform_cat:in", -1, transform_cat(PRECESSION, NOVAS_JD_B1950, NULL, NOVAS_JD_J2000, "FK5", &c))) n++;
   if(check("transform_cat:out", -1, transform_cat(PRECESSION, NOVAS_JD_B1950, &c, NOVAS_JD_J2000, "FK5", NULL))) n++;
@@ -253,7 +270,7 @@ static int test_transform_cat() {
 }
 
 static int test_transform_hip() {
-  cat_entry c = {};
+  cat_entry c = CAT_ENTRY_INIT;
   int n = 0;
 
   if(check("transform_hip:in", -1, transform_hip(NULL, &c))) n++;
@@ -275,7 +292,6 @@ static int test_ephemeris() {
   if(check("ephemeris:pos", -1, ephemeris(tdb, &ceres, NOVAS_BARYCENTER, NOVAS_FULL_ACCURACY, NULL, v))) n++;
   if(check("ephemeris:vel", -1, ephemeris(tdb, &ceres, NOVAS_BARYCENTER, NOVAS_FULL_ACCURACY, p, NULL))) n++;
   if(check("ephemeris:pos+vel", -1, ephemeris(tdb, &ceres, NOVAS_BARYCENTER, NOVAS_FULL_ACCURACY, NULL, NULL))) n++;
-  if(check("ephemeris:pos=vel", -1, ephemeris(tdb, &ceres, NOVAS_BARYCENTER, NOVAS_FULL_ACCURACY, p, p))) n++;
   if(check("ephemeris:origin", 1, ephemeris(tdb, &ceres, -1, NOVAS_FULL_ACCURACY, p, v))) n++;
 
 #ifndef DEFAULT_READEPH
@@ -396,8 +412,8 @@ static int test_set_planet_provider_hp() {
 }
 
 static int test_place_star() {
-  cat_entry c = {};
-  observer loc = {};
+  cat_entry c = CAT_ENTRY_INIT;
+  observer loc = OBSERVER_INIT;
   sky_pos pos;
   int n = 0;
 
@@ -408,8 +424,8 @@ static int test_place_star() {
 }
 
 static int test_place() {
-  object o = {};
-  observer loc = {};
+  object o = NOVAS_OBJECT_INIT;
+  observer loc = OBSERVER_INIT;
   sky_pos pos;
   int n = 0;
 
@@ -422,8 +438,8 @@ static int test_place() {
 }
 
 static int test_radec_planet() {
-  object o = {};
-  observer loc = {};
+  object o = NOVAS_OBJECT_INIT;
+  observer loc = OBSERVER_INIT;
   double ra, dec, dis, rv;
 
   o.type = NOVAS_CATALOG_OBJECT;
@@ -515,8 +531,8 @@ static int test_ecl2equ_vec() {
 
 
 static int test_itrs_to_hor() {
-  on_surface loc = {};
-  double p[3] = {}, az, za;
+  on_surface loc = ON_SURFACE_INIT;
+  double p[3] = {0.0}, az, za;
   int n = 0;
 
   if(check("itrs_to_hor:loc", -1, itrs_to_hor(NULL, p, &az, &za))) n++;
@@ -526,7 +542,7 @@ static int test_itrs_to_hor() {
 }
 
 static int test_hor_to_itrs() {
-  on_surface loc = {};
+  on_surface loc = ON_SURFACE_INIT;
   double p[3];
   int n = 0;
 
@@ -537,7 +553,7 @@ static int test_hor_to_itrs() {
 }
 
 static int test_equ2hor() {
-  on_surface loc;
+  on_surface loc = ON_SURFACE_INIT;
   double az, za, rar, decr;
   int n = 0;
 
@@ -616,12 +632,12 @@ static int test_wobble() {
 }
 
 static int test_terra() {
-  on_surface loc = {};
+  on_surface loc = ON_SURFACE_INIT;
   double p[3], v[3];
   int n = 0;
 
   if(check("terra:loc", -1, terra(NULL, 0.0, p, v))) n++;
-  if(check("terra:same", -1, terra(&loc, 0.0, p, p))) n++;
+  if(check("terra:pos+vel", -1, terra(&loc, 0.0, NULL, NULL))) n++;
 
   return n;
 }
@@ -667,7 +683,7 @@ static int test_proper_motion() {
 }
 
 static int test_bary2obs() {
-  double p[3] = {}, po[3] = {}, out[3], lt;
+  double p[3] = {0.0}, po[3] = {0.0}, out[3], lt;
   int n = 0;
 
   if(check("bary2obs:pos", -1, bary2obs(NULL, po, out, &lt))) n++;
@@ -679,12 +695,12 @@ static int test_bary2obs() {
 
 static int test_geo_posvel() {
   observer o;
-  double p[3], v[3];
+  double p[3] = {0.0}, v[3] = {0.0};
   int n = 0;
 
   o.where = NOVAS_OBSERVER_ON_EARTH;
   if(check("geo_posvel:loc", -1, geo_posvel(0.0, 0.0, NOVAS_FULL_ACCURACY, NULL, p, v))) n++;
-  if(check("geo_posvel:same", -1, geo_posvel(0.0, 0.0, NOVAS_FULL_ACCURACY, &o, p, p))) n++;
+  if(check("geo_posvel:pos+vel", -1, geo_posvel(0.0, 0.0, NOVAS_FULL_ACCURACY, &o, NULL, NULL))) n++;
   if(check("geo_posvel:accuracy", 1, geo_posvel(0.0, 0.0, -1, &o, p, v))) n++;
 
   o.where = -1;
@@ -704,9 +720,6 @@ static int test_light_time2() {
   if(check("light_time2:tout", -1, light_time2(0.0, &o, pos, 0.0, NOVAS_FULL_ACCURACY, p, v, NULL))) n++;
   if(check("light_time2:object", -1, light_time2(0.0, NULL, pos, 0.0, NOVAS_FULL_ACCURACY, p, v, &t))) n++;
   if(check("light_time2:pos", -1, light_time2(0.0, &o, NULL, 0.0, NOVAS_FULL_ACCURACY, p, v, &t))) n++;
-  if(check("light_time2:same1", -1, light_time2(0.0, &o, pos, 0.0, NOVAS_FULL_ACCURACY, pos, v, &t))) n++;
-  if(check("light_time2:same2", -1, light_time2(0.0, &o, pos, 0.0, NOVAS_FULL_ACCURACY, p, pos, &t))) n++;
-  if(check("light_time2:same3", -1, light_time2(0.0, &o, pos, 0.0, NOVAS_FULL_ACCURACY, p, p, &t))) n++;
 
   novas_inv_max_iter = 0;
   if(check("light_time2:converge", 1, light_time2(0.0, &o, pos, 0.0, NOVAS_FULL_ACCURACY, p, v, &t))) n++;
@@ -811,13 +824,12 @@ static int test_cio_ra() {
 }
 
 static int test_starvectors() {
-  cat_entry star;
+  cat_entry star = CAT_ENTRY_INIT;
   double p[3], v[3];
   int n = 0;
 
   if(check("starvectors:star", -1, starvectors(NULL, p, v))) n++;
-  if(check("starvectors:pos+vel", -1, starvectors(NULL, NULL, NULL))) n++;
-  if(check("starvectors:pos=vel", -1, starvectors(&star, p, p))) n++;
+  if(check("starvectors:pos+vel", -1, starvectors(&star, NULL, NULL))) n++;
 
   return n;
 }
@@ -831,7 +843,7 @@ static int test_radec2vector() {
 }
 
 static int test_vector2radec() {
-  double p[3] = {}, ra, dec;
+  double p[3] = {0.0}, ra, dec;
   int n = 0;
 
   if(check("vector2radec:vec", -1, vector2radec(NULL, &ra, &dec))) n++;
@@ -902,7 +914,7 @@ static int test_precession() {
 
 static int test_rad_vel() {
   object o;
-  double p[3] = {}, v[3] = {}, vo[3] = {}, rv;
+  double p[3] = {0.0}, v[3] = {0.0}, vo[3] = {0.0}, rv;
   int n = 0;
 
   o.type = NOVAS_PLANET;
@@ -913,14 +925,19 @@ static int test_rad_vel() {
   if(check("rad_vel:vobs", -1, rad_vel(&o, p, v, NULL, 1.0, 1.0, 1.0, &rv))) n++;
   if(check("rad_vel:out", -1, rad_vel(&o, p, v, vo, 1.0, 1.0, 1.0, NULL))) n++;
 
+  if(check_nan("rad_vel2:emit", rad_vel2(&o, NULL, v, p, vo, 1.0, 1.0, 1.0))) n++;
+  if(check_nan("rad_vel2:det", rad_vel2(&o, p, v, NULL, vo, 1.0, 1.0, 1.0))) n++;
+
   o.type = -1;
   if(check("rad_vel", -1, rad_vel(&o, p, v, vo, 1.0, 1.0, 1.0, &rv))) n++;
 
   return n;
 }
 
+
+
 static int test_aberration() {
-  double p[3] = {1.0}, v[3] = {};
+  double p[3] = {1.0}, v[3] = {0.0};
   int n = 0;
 
   if(check("aberration:pos", -1, aberration(NULL, v, 0.0, p))) n++;
@@ -931,7 +948,7 @@ static int test_aberration() {
 }
 
 static int test_grav_vec() {
-  double p[3] = {2.0}, po[3] = {0.0, 1.0}, pb[3] = {};
+  double p[3] = {2.0}, po[3] = {0.0, 1.0}, pb[3] = {0.0};
   int n = 0;
 
   if(check("grav_vec:pos", -1, grav_vec(NULL, po, pb, 1.0, p))) n++;
@@ -974,7 +991,7 @@ static int test_grav_undef() {
 }
 
 static int test_grav_init_planets() {
-  novas_planet_bundle planets = {};
+  novas_planet_bundle planets = NOVAS_PLANET_BUNDLE_INIT;
   double p[3] = {2.0};
   int n = 0;
 
@@ -985,8 +1002,8 @@ static int test_grav_init_planets() {
 }
 
 static int test_grav_planets() {
-  novas_planet_bundle planets = {};
-  double p[3] = {2.0}, po[3] = {0.0, 1.0}, out[3] = {};
+  novas_planet_bundle planets = NOVAS_PLANET_BUNDLE_INIT;
+  double p[3] = {2.0}, po[3] = {0.0, 1.0}, out[3] = {0.0};
   int n = 0;
 
   if(check("grav_planets:pos_src", -1, grav_planets(NULL, po, &planets, out))) n++;
@@ -999,8 +1016,8 @@ static int test_grav_planets() {
 
 static int test_grav_undo_planets() {
   extern int novas_inv_max_iter;
-  novas_planet_bundle planets = {};
-  double p[3] = {2.0}, po[3] = {0.0, 1.0}, out[3] = {};
+  novas_planet_bundle planets = NOVAS_PLANET_BUNDLE_INIT;
+  double p[3] = {2.0}, po[3] = {0.0, 1.0}, out[3] = {0.0};
   int n = 0;
 
   if(check("grav_undo_planets:pos_app", -1, grav_undo_planets(NULL, po, &planets, out))) n++;
@@ -1068,8 +1085,6 @@ static int test_obs_posvel() {
 
   if(check("obs_posvel:obs", -1, obs_posvel(NOVAS_JD_J2000, 0.0, NOVAS_REDUCED_ACCURACY, NULL, NULL, NULL, &x, NULL))) n++;
   if(check("obs_posvel:obs:pos+vel", -1, obs_posvel(NOVAS_JD_J2000, 0.0, NOVAS_REDUCED_ACCURACY, &obs, NULL, NULL, NULL, NULL))) n++;
-  if(check("obs_posvel:obs:pos=vel", -1, obs_posvel(NOVAS_JD_J2000, 0.0, NOVAS_REDUCED_ACCURACY, &obs, NULL, NULL, &x, &x))) n++;
-
   obs.where = -1;
   if(check("obs_posvel:obs:where:-1", -1, obs_posvel(NOVAS_JD_J2000, 0.0, NOVAS_REDUCED_ACCURACY, &obs, NULL, NULL, &x, NULL))) n++;
 
@@ -1080,7 +1095,7 @@ static int test_obs_posvel() {
 }
 
 static int test_time() {
-  novas_timespec time = {};
+  novas_timespec time = NOVAS_TIMESPEC_INIT;
   int n = 0;
   long ijd = 0;
 
@@ -1128,7 +1143,7 @@ static double switching_refraction(double jd_tt, const on_surface *loc, enum nov
 
 static int test_refraction() {
   int n = 0;
-  on_surface obs = {};
+  on_surface obs = ON_SURFACE_INIT;
 
   if(check_nan("stardard_refraction:loc", novas_standard_refraction(NOVAS_JD_J2000, NULL, NOVAS_REFRACT_OBSERVED, 10.0))) n++;
   if(check_nan("stardard_refraction:type:-2", novas_standard_refraction(NOVAS_JD_J2000, &obs, -2, 10.0))) n++;
@@ -1164,9 +1179,9 @@ static int test_refraction() {
 
 static int test_make_frame() {
   int n = 0;
-  novas_timespec ts = {};
-  observer obs = {};
-  novas_frame frame = {};
+  novas_timespec ts = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT;
 
   if(check("make_frame:obs", -1, novas_make_frame(NOVAS_REDUCED_ACCURACY, NULL, &ts, 0.0, 0.0, &frame))) n++;
   if(check("make_frame:time", -1, novas_make_frame(NOVAS_REDUCED_ACCURACY, &obs, NULL, 0.0, 0.0, &frame))) n++;
@@ -1185,9 +1200,9 @@ static int test_make_frame() {
 
 static int test_change_observer() {
   int n = 0;
-  novas_timespec ts = {};
-  observer obs = {};
-  novas_frame frame = {}, out = {};
+  novas_timespec ts = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT, out = NOVAS_FRAME_INIT;
 
   make_observer_at_geocenter(&obs);
   novas_set_time(NOVAS_TT, NOVAS_JD_J2000, 32, 0.0, &ts);
@@ -1206,10 +1221,10 @@ static int test_change_observer() {
 
 static int test_make_transform() {
   int n = 0;
-  novas_timespec ts = {};
-  observer obs = {};
-  novas_frame frame = {};
-  novas_transform T = {};
+  novas_timespec ts = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT;
+  novas_transform T = NOVAS_TRANSFORM_INIT;
 
   make_observer_at_geocenter(&obs);
   novas_set_time(NOVAS_TT, NOVAS_JD_J2000, 32, 0.0, &ts);
@@ -1231,11 +1246,11 @@ static int test_make_transform() {
 
 static int test_geom_posvel() {
   int n = 0;
-  novas_timespec ts = {};
-  observer obs = {};
-  novas_frame frame = {};
-  object o = {};
-  double pos[3] = {}, vel[3] = {};
+  novas_timespec ts = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT;
+  object o = NOVAS_SUN_INIT;
+  double pos[3] = {0.0}, vel[3] = {0.0};
 
   make_observer_at_geocenter(&obs);
   novas_set_time(NOVAS_TT, NOVAS_JD_J2000, 32, 0.0, &ts);
@@ -1248,7 +1263,6 @@ static int test_geom_posvel() {
 
   if(check("geom_posvel:object", -1, novas_geom_posvel(NULL, &frame, NOVAS_ICRS, pos, vel))) n++;
   if(check("geom_posvel:pos+vel", -1, novas_geom_posvel(&o, &frame, NOVAS_ICRS, NULL, NULL))) n++;
-  if(check("geom_posvel:pos=vel", -1, novas_geom_posvel(&o, &frame, NOVAS_ICRS, pos, pos))) n++;
   if(check("geom_posvel:sys:-1", -1, novas_geom_posvel(&o, &frame, -1, pos, vel))) n++;
   if(check("geom_posvel:sys:hi", -1, novas_geom_posvel(&o, &frame, NOVAS_REFERENCE_SYSTEMS, pos, vel))) n++;
 
@@ -1269,11 +1283,11 @@ static int test_geom_posvel() {
 
 static int test_sky_pos() {
   int n = 0;
-  novas_timespec ts = {};
-  observer obs = {};
-  novas_frame frame = {};
-  object o = {};
-  sky_pos out = {};
+  novas_timespec ts = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT;
+  object o = NOVAS_SSB_INIT;
+  sky_pos out = SKY_POS_INIT;
 
   make_observer_at_geocenter(&obs);
   novas_set_time(NOVAS_TT, NOVAS_JD_J2000, 32, 0.0, &ts);
@@ -1303,10 +1317,10 @@ static int test_sky_pos() {
 
 static int test_app_to_geom() {
   extern int novas_inv_max_iter;
-  novas_timespec ts = {};
-  observer obs = {};
-  novas_frame frame = {};
-  double pos[3] = {};
+  novas_timespec ts = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT;
+  double pos[3] = {0.0};
   int n = 0;
 
   make_observer_at_geocenter(&obs);
@@ -1329,11 +1343,11 @@ static int test_app_to_geom() {
 }
 
 static int test_geom_to_app() {
-  novas_timespec ts = {};
-  observer obs = {};
-  novas_frame frame = {};
-  sky_pos out = {};
-  double pos[3] = {};
+  novas_timespec ts = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT;
+  sky_pos out = SKY_POS_INIT;
+  double pos[3] = {0.0};
   int n = 0;
 
   make_observer_on_surface(1.0, 2.0, 3.0, 4.0, 1001.0, &obs);
@@ -1359,9 +1373,9 @@ static int test_geom_to_app() {
 }
 
 static int test_app_to_hor() {
-  novas_timespec ts = {};
-  observer obs = {};
-  novas_frame frame = {};
+  novas_timespec ts = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT;
   double az, el;
   int n = 0;
 
@@ -1385,9 +1399,9 @@ static int test_app_to_hor() {
 }
 
 static int test_hor_to_app() {
-  novas_timespec ts = {};
-  observer obs = {};
-  novas_frame frame = {};
+  novas_timespec ts = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT;
   double ra, dec;
   int n = 0;
 
@@ -1412,11 +1426,11 @@ static int test_hor_to_app() {
 
 
 static int test_transform_vector() {
-  novas_timespec ts = {};
-  observer obs = {};
-  novas_frame frame = {};
-  novas_transform T = {};
-  double pos[3] = {};
+  novas_timespec ts = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT;
+  novas_transform T = NOVAS_TRANSFORM_INIT;
+  double pos[3] = {0.0};
   int n = 0;
 
   make_observer_at_geocenter(&obs);
@@ -1434,11 +1448,11 @@ static int test_transform_vector() {
 }
 
 static int test_transform_sky_pos() {
-  novas_timespec ts = {};
-  observer obs = {};
-  novas_frame frame = {};
-  novas_transform T = {};
-  sky_pos pos = {};
+  novas_timespec ts = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT;
+  novas_transform T = NOVAS_TRANSFORM_INIT;
+  sky_pos pos = SKY_POS_INIT;
   int n = 0;
 
 
@@ -1457,7 +1471,7 @@ static int test_transform_sky_pos() {
 }
 
 static int test_inv_transform() {
-  novas_transform T = {};
+  novas_transform T = NOVAS_TRANSFORM_INIT;
   int n = 0;
 
   if(check("invert_transform:in", -1, novas_invert_transform(NULL, &T))) n++;
@@ -1547,8 +1561,8 @@ static int test_planet_for_name() {
 
 static int test_make_orbital_object() {
   int n = 0;
-  novas_orbital orbit = {};
-  object body = {};
+  novas_orbital orbit = NOVAS_ORBIT_INIT;
+  object body = NOVAS_OBJECT_INIT;
 
   if(check("make_orbital_object:orbit", -1, make_orbital_object("blah", -1, NULL, &body))) n++;
   if(check("make_orbital_object:body", -1, make_orbital_object("blah", -1, &orbit, NULL))) n++;
@@ -1559,14 +1573,13 @@ static int test_make_orbital_object() {
 
 static int test_orbit_posvel() {
   int n = 0;
-  double pos[3] = {}, vel[3] = {};
+  double pos[3] = {0.0}, vel[3] = {0.0};
   int saved = novas_inv_max_iter;
   novas_orbital orbit = NOVAS_ORBIT_INIT;
 
   orbit.a = 1.0;
 
   if(check("set_obsys_pole:orbit", -1, novas_orbit_posvel(0.0, NULL, NOVAS_REDUCED_ACCURACY, pos, vel))) n++;
-  if(check("set_obsys_pole:pos=vel", -1, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, pos, pos))) n++;
   if(check("set_obsys_pole:pos=vel:NULL", -1, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, NULL, NULL))) n++;
   if(check("set_obsys_pole:accuracy:-1", -1, novas_orbit_posvel(0.0, &orbit, -1, pos, vel))) n++;
   if(check("set_obsys_pole:accuracy:2", -1, novas_orbit_posvel(0.0, &orbit, 2, pos, vel))) n++;
@@ -1603,6 +1616,463 @@ static int test_set_obsys_pole() {
   return n;
 }
 
+static int test_hms_hours() {
+  int n = 0;
+
+  if(check_nan("hms_hours:null", novas_hms_hours(NULL))) n++;
+  if(check_nan("hms_hours:empty", novas_hms_hours(""))) n++;
+  if(check_nan("hms_hours:empty", novas_hms_hours(""))) n++;
+  if(check_nan("hms_hours:few", novas_hms_hours("12"))) n++;
+  if(check_nan("hms_hours:dms", novas_hms_hours("12d 39m 33.0"))) n++;
+  if(check_nan("hms_hours:sep", novas_hms_hours("12,39,33.0"))) n++;
+  if(check_nan("hms_hours:min:neg", novas_hms_hours("12 -1 33.0"))) n++;
+  if(check_nan("hms_hours:min:60", novas_hms_hours("12 60 33.0"))) n++;
+  if(check_nan("hms_hours:sec:neg", novas_hms_hours("12 39 -0.1"))) n++;
+  if(check_nan("hms_hours:min:60", novas_hms_hours("12 39 60.0"))) n++;
+
+  return n;
+}
+
+static int test_dms_degrees() {
+  int n = 0;
+
+  if(check_nan("dms_degrees:null", novas_dms_degrees(NULL))) n++;
+  if(check_nan("dms_degrees:empty", novas_dms_degrees(""))) n++;
+  if(check_nan("dms_degrees:empty", novas_dms_degrees(""))) n++;
+  if(check_nan("dms_degrees:few", novas_dms_degrees("122"))) n++;
+  if(check_nan("dms_degrees:hms", novas_dms_degrees("122h 39m 33.0"))) n++;
+  if(check_nan("dms_degrees:sep", novas_dms_degrees("122,39,33.0"))) n++;
+  if(check_nan("dms_degrees:min:neg", novas_dms_degrees("122 -1 33.0"))) n++;
+  if(check_nan("dms_degrees:min:60", novas_dms_degrees("122 60 33.0"))) n++;
+  if(check_nan("dms_degrees:sec:neg", novas_dms_degrees("122 39 -0.1"))) n++;
+  if(check_nan("dms_degrees:min:60", novas_dms_degrees("122 39 60.0"))) n++;
+
+  return n;
+}
+
+static int test_parse_hours() {
+  int n = 0;
+  char *tail;
+
+  if(check_nan("parse_hours:null", novas_parse_hours(NULL, &tail))) n++;
+  if(check_nan("parse_hours:blah", novas_parse_hours("blah", &tail))) n++;
+
+  return n;
+}
+
+static int test_parse_degrees() {
+  int n = 0;
+  char *tail;
+
+  if(check_nan("parse_degrees:null", novas_parse_degrees(NULL, &tail))) n++;
+  if(check_nan("parse_degrees:blah", novas_parse_degrees("blah", &tail))) n++;
+
+  return n;
+}
+
+static int test_str_hours() {
+  int n = 0;
+
+  if(check_nan("str_hours:null", novas_str_hours(NULL))) n++;
+  if(check_nan("str_hours:blah", novas_str_hours("blah"))) n++;
+
+  return n;
+}
+
+static int test_str_degrees() {
+  int n = 0;
+
+  if(check_nan("str_degrees:null", novas_str_degrees(NULL))) n++;
+  if(check_nan("str_degrees:blah", novas_str_degrees("blah"))) n++;
+
+  return n;
+}
+
+
+static int test_helio_dist() {
+  int n = 0;
+  double rate = 0.0;
+  object star = NOVAS_OBJECT_INIT;
+  object jupiter = NOVAS_JUPITER_INIT;
+
+  star.type = NOVAS_CATALOG_OBJECT;
+
+  if(check_nan("helio_dist:null", novas_helio_dist(JD_J2000, NULL, NULL))) n++;
+  if(check_nan("helio_dist:cat_object", novas_helio_dist(JD_J2000, &star, NULL))) n++;
+  if(check_nan("helio_dist:null:rate", novas_helio_dist(JD_J2000, NULL, &rate))) n++;
+  if(check_nan("helio_dist:null:rate:nan", rate)) n++;
+  if(check_nan("helio_dist:cat_object:rate", novas_helio_dist(JD_J2000, &star, &rate))) n++;
+  if(check_nan("helio_dist:cat_object:rate:nan", rate)) n++;
+  if(check_nan("helio_dist:source:invalid", novas_helio_dist(JD_J2000, &jupiter, &rate))) n++;
+
+  return n;
+}
+
+static int test_frame_lst() {
+  int n = 0;
+  novas_timespec time = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT;
+
+  if(check_nan("frame_lst:frame:null", novas_frame_lst(NULL))) n++;
+  if(check_nan("frame_lst:frame:init", novas_frame_lst(&frame))) n++;
+
+  novas_set_time(NOVAS_TDB, NOVAS_JD_J2000, 32.0, 0.0, &time);
+  make_observer_at_geocenter(&obs);
+  novas_make_frame(NOVAS_REDUCED_ACCURACY, &obs, &time, 0.0, 0.0, &frame);
+
+  if(check_nan("frame_lst:obs:invalid", novas_frame_lst(&frame))) n++;
+
+  return n;
+}
+
+static int test_rise_set() {
+  int n = 0;
+  object sun = NOVAS_SUN_INIT;
+  novas_timespec time = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT;
+  int save = novas_inv_max_iter;
+
+  if(check_nan("rise_set:rises_above:frame:null", novas_rises_above(0.0, &sun, NULL, NULL))) n++;
+  if(check_nan("rise_set:rises_above:frame:init", novas_rises_above(0.0, &sun, &frame, NULL))) n++;
+  if(check_nan("rise_set:sets_below:frame:null", novas_sets_below(0.0, &sun, NULL, NULL))) n++;
+  if(check_nan("rise_set:sets_below:frame:init", novas_sets_below(0.0, &sun, &frame, NULL))) n++;
+  if(check_nan("rise_set:transit_time:frame:null", novas_transit_time(&sun, NULL))) n++;
+  if(check_nan("rise_set:transit_time:frame:init", novas_transit_time(&sun, &frame))) n++;
+
+  // noon (near transit)
+  novas_set_time(NOVAS_TDB, NOVAS_JD_J2000, 32.0, 0.0, &time);
+  make_observer_on_surface(0.0, 0.0, 0.0, 0.0, 0.0, &obs);
+  if(check("rise_set:make_frame", 0, novas_make_frame(NOVAS_REDUCED_ACCURACY, &obs, &time, 0.0, 0.0, &frame))) n++;
+
+  if(check_nan("rise_set:rises_above:source:null", novas_rises_above(0.0, NULL, &frame, NULL))) n++;
+  if(check_nan("rise_set:sets_below:source:null", novas_sets_below(0.0, NULL, &frame, NULL))) n++;
+  if(check_nan("rise_set:transit_time:source:null", novas_transit_time(NULL, &frame))) n++;
+
+  make_observer_on_surface(60.0, 0.0, 0.0, 0.0, 0.0, &obs);
+  novas_change_observer(&frame, &obs, &frame);
+  if(check_nan("rise_set:rises_above:source:null", novas_rises_above(31.0, &sun, &frame, NULL))) n++;
+  if(check_nan("rise_set:sets_below:source:null", novas_sets_below(31.0, &sun, &frame, NULL))) n++;
+
+  novas_inv_max_iter = 0;
+  if(check_nan("rise_set:rises_above:noconv", novas_rises_above(0.0, &sun, &frame, NULL))) n++;
+  if(check_nan("rise_set:sets_below:noconv", novas_rises_above(0.0, &sun, &frame, NULL))) n++;
+  novas_inv_max_iter = save;
+
+  return n;
+}
+
+static int test_tracks() {
+  int n = 0;
+  object sun = NOVAS_SUN_INIT;
+  novas_timespec time = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT;
+  novas_track track = NOVAS_TRACK_INIT;
+  double vel[3] = {0.0}, x;
+
+  if(check("equ_track:frame:null", -1, novas_equ_track(&sun, NULL, 1000.0, &track))) n++;
+  if(check("equ_track:frame:init", -1, novas_equ_track(&sun, &frame, 1000.0, &track))) n++;
+  if(check("hor_track:frame:null", -1, novas_hor_track(&sun, NULL, NULL, &track))) n++;
+  if(check("hor_track:frame:init", -1, novas_hor_track(&sun, &frame, NULL, &track))) n++;
+
+  // noon (near transit)
+  novas_set_time(NOVAS_TDB, NOVAS_JD_J2000, 32.0, 0.0, &time);
+  make_observer_on_surface(0.0, 0.0, 0.0, 0.0, 0.0, &obs);
+  if(check("rise_set:make_frame", 0, novas_make_frame(NOVAS_REDUCED_ACCURACY, &obs, &time, 0.0, 0.0, &frame))) n++;
+
+  if(check("equ_track:source:null", -1, novas_equ_track(NULL, &frame, 1000.0, &track))) n++;
+  if(check("equ_track:track:null", -1, novas_equ_track(&sun, &frame, 1000.0, NULL))) n++;
+  if(check("hor_track:source:null", -1, novas_hor_track(NULL, &frame, NULL, &track))) n++;
+  if(check("hor_track:track:null", -1, novas_hor_track(&sun, &frame, NULL, NULL))) n++;
+
+  if(check("hor_track:make_airborne_observer", 0, make_airborne_observer(&obs.on_surf, vel, &obs))) n++;
+  if(check("rise_set:make_frame:airborne", 0, novas_make_frame(NOVAS_REDUCED_ACCURACY, &obs, &time, 0.0, 0.0, &frame))) n++;
+  if(check("hor_track:track:null", 0, novas_hor_track(&sun, &frame, NULL, &track))) n++;
+
+  make_observer_at_geocenter(&obs);
+  if(check("rise_set:make_frame:geocenter", 0, novas_make_frame(NOVAS_REDUCED_ACCURACY, &obs, &time, 0.0, 0.0, &frame))) n++;
+  if(check("hor_track:track:null", -1, novas_hor_track(&sun, &frame, NULL, &track))) n++;
+
+  if(check("track_pos:track:null", -1, novas_track_pos(NULL, &time, &x, NULL, NULL, NULL))) n++;
+  if(check("track_pos:time:null", -1, novas_track_pos(&track, NULL, &x, NULL, NULL, NULL))) n++;
+
+  return n;
+}
+
+static int test_solar_illum() {
+  int n = 0;
+  object earth = NOVAS_EARTH_INIT;
+  object jupiter = NOVAS_JUPITER_INIT;
+  novas_timespec time = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT;
+  double pos[3] = {1.0, 1.0, 0.0}, vel[3] = {0.0};
+
+  make_solar_system_observer(pos, vel, &obs);
+  novas_set_time(NOVAS_TDB, NOVAS_JD_J2000, 32.0, 0.0, &time);
+
+  if(check_nan("solar_illum:frame:null", novas_solar_illum(&earth, NULL))) n++;
+  if(check_nan("solar_illum:frame:init", novas_solar_illum(&earth, &frame))) n++;
+
+  if(check("solar_illum:make_frame", 0, novas_make_frame(NOVAS_REDUCED_ACCURACY, &obs, &time, 0.0, 0.0, &frame))) n++;
+  if(check_nan("solar_illum:source:null", novas_solar_illum(NULL, &frame))) n++;
+  if(check_nan("solar_illum:source:invalid", novas_solar_illum(&jupiter, &frame))) n++;
+
+  return n;
+}
+
+static int test_object_sep() {
+  int n = 0;
+  object sun = NOVAS_SUN_INIT;
+  object earth = NOVAS_EARTH_INIT;
+  object jupiter = NOVAS_JUPITER_INIT;
+  novas_timespec time = NOVAS_TIMESPEC_INIT;
+  observer obs = OBSERVER_INIT;
+  novas_frame frame = NOVAS_FRAME_INIT;
+
+  make_observer_at_geocenter(&obs);
+  novas_set_time(NOVAS_TDB, NOVAS_JD_J2000, 32.0, 0.0, &time);
+
+  if(check_nan("object_sep:frame:null", novas_object_sep(&sun, &sun, NULL))) n++;
+  if(check_nan("object_sep:frame:init", novas_object_sep(&sun, &sun, &frame))) n++;
+
+  if(check("object_sep:make_frame", 0, novas_make_frame(NOVAS_REDUCED_ACCURACY, &obs, &time, 0.0, 0.0, &frame))) n++;
+
+  if(check_nan("object_sep:a:null", novas_object_sep(NULL, &sun, &frame))) n++;
+  if(check_nan("object_sep:b:null", novas_object_sep(&sun, NULL, &frame))) n++;
+
+  if(check_nan("object_sep:a:invalid", novas_object_sep(&jupiter, &sun, &frame))) n++;
+  if(check_nan("object_sep:b:invalid", novas_object_sep(&sun, &jupiter, &frame))) n++;
+
+  if(check_nan("object_sep:a=obs", novas_object_sep(&earth, &sun, &frame))) n++;
+  if(check_nan("object_sep:b=obs", novas_object_sep(&sun, &earth, &frame))) n++;
+  if(check_nan("object_sep:a=b=obs", novas_object_sep(&earth, &earth, &frame))) n++;
+
+  if(check_nan("object_sep:sun_angle", novas_sun_angle(&earth, &frame))) n++;
+  if(check_nan("object_sep:sun_angle", novas_moon_angle(&earth, &frame))) n++;
+
+  return n;
+}
+
+static int test_uvw_to_xyz() {
+  int n = 0;
+  double v[3] = {0.0};
+
+  if(check("uvw_to_xyz:uvw:null", -1, novas_uvw_to_xyz(NULL, 0.0, 0.0, v))) n++;
+  if(check("xyz_to_uvw:xyz:null", -1, novas_uvw_to_xyz(v, 0.0, 0.0, NULL))) n++;
+
+  return n;
+}
+
+static int test_xyz_to_uvw() {
+  int n = 0;
+  double v[3] = {0.0};
+
+  if(check("xyz_to_uvw:xyz:null", -1, novas_xyz_to_uvw(NULL, 0.0, 0.0, v))) n++;
+  if(check("xyz_to_uvw:uvw:null", -1, novas_xyz_to_uvw(v, 0.0, 0.0, NULL))) n++;
+
+  return n;
+}
+
+static int test_julian_date() {
+  int n = 0;
+
+  if(check_nan("julian_date:m:0", julian_date(2000.0, 0, 1, 0.0))) n++;
+  if(check_nan("julian_date:m:13", julian_date(2000.0, 13, 1, 0.0))) n++;
+  if(check_nan("julian_date:d:0", julian_date(2000.0, 2, 0, 0.0))) n++;
+  if(check_nan("julian_date:feb:d:30", julian_date(2000.0, 2, 30, 0.0))) n++;
+
+  return n;
+}
+
+static int test_parse_date() {
+  int n = 0;
+  char *tail = NULL;
+
+  if(check_nan("parse_date:null", novas_parse_date(NULL, &tail))) n++;
+  if(check_nan("parse_date:empty", novas_parse_date("", &tail))) n++;
+
+  if(check_nan("parse_date_time:format:-1", novas_parse_date_format(0, -1, "2025-01-28", &tail))) n++;
+  if(check_nan("parse_date_time:format:3", novas_parse_date_format(0, 3, "2025-01-28", &tail))) n++;
+
+  if(check_nan("parse_date_time:few", novas_parse_date_format(0, NOVAS_YMD, "2025-01", &tail))) n++;
+  if(check_nan("parse_date_time:sep:invalid", novas_parse_date_format(0, 3, "2025$01$28", &tail))) n++;
+
+  if(check_nan("parse_date_time:m:0", novas_parse_date_format(0, NOVAS_YMD, "2025-00-28", &tail))) n++;
+  if(check_nan("parse_date_time:m:13", novas_parse_date_format(0, NOVAS_YMD, "2025-13-28", &tail))) n++;
+  if(check_nan("parse_date_time:d:0", novas_parse_date_format(0, NOVAS_YMD, "2025-2-0", &tail))) n++;
+  if(check_nan("parse_date_time:feb:d:30", novas_parse_date_format(0, NOVAS_YMD, "2025-2-30", &tail))) n++;
+
+  if(check_nan("parse_date_time:m:invalid", novas_parse_date_format(0, NOVAS_YMD, "2025-blah-28", &tail))) n++;
+
+  if(check_nan("parse_date_time:zone:missing", novas_parse_date_format(0, NOVAS_YMD, "2025-01-28 12:50:00+", &tail))) n++;
+  if(check_nan("parse_date_time:zone:bad", novas_parse_date_format(0, NOVAS_YMD, "2025-01-28 12:50:00+:", &tail))) n++;
+  if(check_nan("parse_date_time:zone:bad2", novas_parse_date_format(0, NOVAS_YMD, "2025-01-28 12:50:00+0:", &tail))) n++;
+  if(check_nan("parse_date_time:zone:hours:24", novas_parse_date_format(0, NOVAS_YMD, "2025-01-28 12:50:00+2400", &tail))) n++;
+  if(check_nan("parse_date_time:zone:mins:60", novas_parse_date_format(0, NOVAS_YMD, "2025-01-28 12:50:00+0260", &tail))) n++;
+  if(check_nan("parse_date_time:zone:mins:incomplete", novas_parse_date_format(0, NOVAS_YMD, "2025-01-28 12:50:00+020", &tail))) n++;
+
+  return n;
+}
+
+static int test_parse_iso_date() {
+  int n = 0;
+  char *tail = NULL;
+
+  if(check_nan("parse_iso_date:null", novas_parse_iso_date(NULL, &tail))) n++;
+  if(check_nan("parse_iso_date:empty", novas_parse_iso_date("", &tail))) n++;
+
+  return n;
+}
+
+static int test_date() {
+  int n = 0;
+
+  if(check_nan("date:null", novas_date(NULL))) n++;
+
+  return n;
+}
+
+static int test_date_scale() {
+  int n = 0;
+  enum novas_timescale scale;
+
+  if(check_nan("date_scale:date:null", novas_date_scale(NULL, &scale))) n++;
+  if(check("date_scale:date:null:scale", -1, scale)) n++;
+  if(check_nan("date_scale:scale:null", novas_date_scale("2025-03-01T11:08:38.338+0200", NULL))) n++;
+
+  return n;
+}
+
+static int test_iso_timestamp() {
+  int n = 0;
+  char buf[30] = {'\0'};
+  novas_timespec time = NOVAS_TIMESPEC_INIT;
+
+  if(check("iso_timestamp:time:null", -1, novas_iso_timestamp(NULL, buf, sizeof(buf)))) n++;
+  if(check("iso_timestamp:buf:null", -1, novas_iso_timestamp(&time, NULL, sizeof(buf)))) n++;
+  if(check("iso_timestamp:len:0", -1, novas_iso_timestamp(&time, buf, 0))) n++;
+
+  return n;
+}
+
+static int test_timestamp() {
+  int n = 0;
+  char buf[30] = {'\0'};
+  novas_timespec time = NOVAS_TIMESPEC_INIT;
+
+  if(check("timestamp:time:null", -1, novas_timestamp(NULL, NOVAS_UTC, buf, sizeof(buf)))) n++;
+  if(check("timestamp:time:scale:-1", -1, novas_timestamp(NULL, -1, buf, sizeof(buf)))) n++;
+  if(check("timestamp:time:scale:hi", -1, novas_timestamp(NULL, NOVAS_TIMESCALES, buf, sizeof(buf)))) n++;
+  if(check("timestamp:buf:null", -1, novas_timestamp(&time, NOVAS_UTC, NULL, sizeof(buf)))) n++;
+  if(check("timestamp:len:0", -1, novas_timestamp(&time, NOVAS_UTC, buf, 0))) n++;
+
+  return n;
+}
+
+static int test_timescale_for_string() {
+  int n = 0;
+
+  if(check("timescale_for_string:null", -1, novas_timescale_for_string(NULL))) n++;
+  if(check("timescale_for_string:empty", -1, novas_timescale_for_string(""))) n++;
+  if(check("timescale_for_string:invalid", -1, novas_timescale_for_string("blah"))) n++;
+
+  return n;
+}
+
+static int test_parse_timescale() {
+  int n = 0;
+  char *tail = NULL;
+
+  if(check("parse_timescale:null", -1, novas_parse_timescale(NULL, &tail))) n++;
+  if(check("parse_timescale:invalid", -1, novas_parse_timescale("blah", &tail))) n++;
+
+  return n;
+}
+
+static int test_print_timescale() {
+  int n = 0;
+  char buf[4] = {'\0'};
+
+  if(check("print_timescale:buf:null", -1, novas_print_timescale(NOVAS_UTC, NULL))) n++;
+  if(check("print_timescale:buf:-1", -1, novas_print_timescale(-1, buf))) n++;
+  if(check("print_timescale:buf:hi", -1, novas_print_timescale(NOVAS_TIMESCALES, buf))) n++;
+
+  return n;
+}
+
+static int test_epoch() {
+  int n = 0;
+
+  if(check_nan("epoch:null", novas_epoch(NULL))) n++;
+  if(check_nan("epoch:empty", novas_epoch(""))) n++;
+  if(check_nan("epoch:blah", novas_epoch("blah"))) n++;
+
+  return n;
+}
+
+static int test_make_cat_object_sys() {
+  int n = 0;
+  cat_entry star = CAT_ENTRY_INIT;
+  object source = NOVAS_OBJECT_INIT;
+
+  if(check("make_cat_object_sys:star:null", -1, make_cat_object_sys(NULL, "ICRS", &source))) n++;
+  if(check("make_cat_object_sys:sys:null", -1, make_cat_object_sys(&star, NULL, &source))) n++;
+  if(check("make_cat_object_sys:source:null", -1, make_cat_object_sys(&star, "ICRS", NULL))) n++;
+  if(check("make_cat_object_sys:sys:bad", -1, make_cat_object_sys(&star, "blah", &source))) n++;
+
+  return n;
+}
+
+static int test_make_redshifted_object_sys() {
+  int n = 0;
+  object source = NOVAS_OBJECT_INIT;
+
+  if(check("make_redshifted_object_sys:sys:null", -1, make_redshifted_object_sys("test", 0.0, 0.0, NULL, 0.0, &source))) n++;
+  if(check("make_redshifted_object_sys:source:null", -1, make_redshifted_object_sys("test", 0.0, 0.0, "ICRS", 0.0, NULL))) n++;
+  if(check("make_redshifted_object_sys:sys:bad", -1, make_redshifted_object_sys("test", 0.0, 0.0, "blah", 0.0, &source))) n++;
+
+  return n;
+}
+
+static int test_planet_ephem_provider() {
+  int n = 0;
+  double pos[3] = {0.0}, vel[3] = {0.0};
+
+  if(check("planet_ephem_provider:none", 1, planet_ephem_provider(NOVAS_JD_J2000, NOVAS_SUN, NOVAS_BARYCENTER, pos, vel))) n++;
+  set_ephem_provider(dummy_ephem);
+
+  if(check("planet_ephem_provider:set", 0, planet_ephem_provider(NOVAS_JD_J2000, NOVAS_SUN, NOVAS_BARYCENTER, pos, vel))) n++;
+  if(check("planet_ephem_provider:planet:-1", -1, planet_ephem_provider(NOVAS_JD_J2000, -1, NOVAS_BARYCENTER, pos, vel))) n++;
+  if(check("planet_ephem_provider:planet:hi", -1, planet_ephem_provider(NOVAS_JD_J2000, NOVAS_PLANETS, NOVAS_BARYCENTER, pos, vel))) n++;
+  if(check("planet_ephem_provider:origin:-1", 1, planet_ephem_provider(NOVAS_JD_J2000, NOVAS_SUN, -1, pos, vel))) n++;
+  if(check("planet_ephem_provider:origin:2", 1, planet_ephem_provider(NOVAS_JD_J2000, NOVAS_SUN, 2, pos, vel))) n++;
+
+  return n;
+}
+
+static int test_print_hms() {
+  int n = 0;
+  char buf[20] = {'\0'};
+
+  if(check("print_hms:buf:null", -1, novas_print_hms(12.0, NOVAS_SEP_COLONS, 3, NULL, sizeof(buf)))) n++;
+  if(check("print_hms:len:0", -1, novas_print_hms(12.0, NOVAS_SEP_COLONS, 3, buf, 0))) n++;
+
+  return n;
+}
+
+static int test_print_dms() {
+  int n = 0;
+  char buf[20] = {'\0'};
+
+  if(check("print_dms:buf:null", -1, novas_print_dms(90.0, NOVAS_SEP_COLONS, 3, NULL, sizeof(buf)))) n++;
+  if(check("print_dms:len:0", -1, novas_print_dms(90.0, NOVAS_SEP_COLONS, 3, buf, 0))) n++;
+
+  return n;
+}
+
 int main() {
   int n = 0;
 
@@ -1741,6 +2211,42 @@ int main() {
   if(test_gcrs_to_mod()) n++;
   if(test_mod_to_gcrs()) n++;
 
+  if(test_hms_hours()) n++;
+  if(test_dms_degrees()) n++;
+  if(test_parse_hours()) n++;
+  if(test_parse_degrees()) n++;
+  if(test_str_hours()) n++;
+  if(test_str_degrees()) n++;
+
+  if(test_helio_dist()) n++;
+
+  if(test_frame_lst()) n++;
+  if(test_rise_set()) n++;
+  if(test_tracks()) n++;
+  if(test_solar_illum()) n++;
+  if(test_object_sep()) n++;
+  if(test_uvw_to_xyz()) n++;
+  if(test_xyz_to_uvw()) n++;
+  if(test_julian_date()) n++;
+  if(test_parse_date()) n++;
+  if(test_parse_iso_date()) n++;
+  if(test_date()) n++;
+  if(test_date_scale()) n++;
+  if(test_iso_timestamp()) n++;
+  if(test_timestamp()) n++;
+  if(test_timescale_for_string()) n++;
+  if(test_parse_timescale()) n++;
+  if(test_print_timescale()) n++;
+
+  if(test_epoch()) n++;
+  if(test_make_cat_object_sys()) n++;
+  if(test_make_redshifted_object_sys()) n++;
+
+  if(test_planet_ephem_provider()) n++;
+
+  if(test_print_hms()) n++;
+  if(test_print_dms()) n++;
+
   if(n) fprintf(stderr, " -- FAILED %d tests\n", n);
   else fprintf(stderr, " -- OK\n");
 
